@@ -59,6 +59,7 @@
 #endif
 
 #define SERVER_UTC_TIME "server_utc_time"
+#define LOCAL_UTC_TIME "local_utc_time"
 #define CACHE_MARKER "populated"
 
 #define EDC_ERROR(_code) e_data_cal_create_error (_code, NULL)
@@ -96,6 +97,8 @@ struct _ECalBackendGroupwisePrivate {
 	/* fields for storing info while offline */
 	gchar *user_email;
 
+	gboolean first_delta_fetch;
+
 	/* A mutex to control access to the private structure for the following */
 	GStaticRecMutex rec_mutex;
 	icaltimezone *default_zone;
@@ -191,6 +194,38 @@ get_element_type (icalcomponent_kind kind)
 
 }
 
+/* TODO Do not replicate this in every backend */
+static icaltimezone *
+resolve_tzid (const gchar *tzid, gpointer user_data)
+{
+	icaltimezone *zone;
+
+	zone = (!strcmp (tzid, "UTC"))
+		? icaltimezone_get_utc_timezone ()
+		: icaltimezone_get_builtin_timezone_from_tzid (tzid);
+
+	if (!zone)
+		zone = e_cal_backend_internal_get_timezone (E_CAL_BACKEND (user_data), tzid);
+
+	return zone;
+}
+
+static void
+put_component_to_store (ECalBackendGroupwise *cbgw,
+			ECalComponent *comp)
+{
+	time_t time_start, time_end;
+	ECalBackendGroupwisePrivate *priv;
+
+	priv = cbgw->priv;
+
+	e_cal_util_get_component_occur_times (comp, &time_start, &time_end,
+				   resolve_tzid, cbgw, priv->default_zone,
+				   e_cal_backend_get_kind (E_CAL_BACKEND (cbgw)));
+
+	e_cal_backend_store_put_component_with_time_range (priv->store, comp, time_start, time_end);
+}
+
 /* Initialy populate the cache from the server */
 static EGwConnectionStatus
 populate_cache (ECalBackendGroupwise *cbgw)
@@ -302,7 +337,7 @@ populate_cache (ECalBackendGroupwise *cbgw)
 					comp_str = e_cal_component_get_as_string (comp);
 					e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), (const gchar *) comp_str);
 					g_free (comp_str);
-					e_cal_backend_store_put_component (priv->store, comp);
+					put_component_to_store (cbgw, comp);
 					g_object_unref (comp);
 				}
 				g_free (progress_string);
@@ -370,15 +405,11 @@ get_deltas (gpointer handle)
 	GList *item_list = NULL, *total_list = NULL, *l;
 	GSList *cache_ids = NULL, *ls;
 	GPtrArray *uid_array = NULL;
-	gchar *time_string = NULL;
-	gchar t_str [26];
+	gchar t_str[26];
+	const gchar *local_utc_time = NULL, *time_string = NULL, *serv_time, *position;
 	gchar *attempts;
-	const gchar *serv_time;
-	const gchar *position;
-
+	icaltimetype current;
 	EGwFilter *filter;
-	gint time_interval;
-	icaltimetype temp;
 	gboolean done = FALSE;
 	gint cursor = 0;
 	struct tm tm;
@@ -403,26 +434,11 @@ get_deltas (gpointer handle)
 
 	serv_time = e_cal_backend_store_get_key_value (store, SERVER_UTC_TIME);
 	if (serv_time) {
-		icaltimetype tmp;
-		g_strlcpy (t_str, e_cal_backend_store_get_key_value (store, SERVER_UTC_TIME), 26);
-		if (!*t_str || !strcmp (t_str, "")) {
-			/* FIXME: When time-stamp is crashed, getting changes from current time */
-			g_warning ("\n\a Could not get the correct time stamp. \n\a");
-			tmp = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
-			current_time = icaltime_as_timet_with_zone (tmp, icaltimezone_get_utc_timezone ());
-			gmtime_r (&current_time, &tm);
-			strftime (t_str, 26, "%Y-%m-%dT%H:%M:%SZ", &tm);
-		}
-	} else {
-		icaltimetype tmp;
-		/* FIXME: When time-stamp is crashed, getting changes from current time */
-		g_warning ("\n\a Could not get the correct time stamp. \n\a");
-		tmp = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
-		current_time = icaltime_as_timet_with_zone (tmp, icaltimezone_get_utc_timezone ());
-		gmtime_r (&current_time, &tm);
-		strftime (t_str, 26, "%Y-%m-%dT%H:%M:%SZ", &tm);
-	}
-	time_string = g_strdup (t_str);
+		time_string = e_cal_backend_store_get_key_value (store, SERVER_UTC_TIME);
+		if (!time_string || !*time_string)
+			time_string = e_gw_connection_get_server_time (priv->cnc);
+	} else
+		time_string = e_gw_connection_get_server_time (priv->cnc);
 
 	filter = e_gw_filter_new ();
 	/* Items modified after the time-stamp */
@@ -503,7 +519,7 @@ get_deltas (gpointer handle)
 			g_free (modif_comp_str);
 			g_free (cache_comp_str);
 			cache_comp_str = NULL;
-			e_cal_backend_store_put_component (store, modified_comp);
+			put_component_to_store (cbgw, modified_comp);
 		}
 
 		e_cal_component_free_icaltimetype (tt);
@@ -518,26 +534,44 @@ get_deltas (gpointer handle)
 	}
 	e_cal_backend_store_thaw_changes (store);
 
-	temp = icaltime_from_string (time_string);
-	current_time = icaltime_as_timet_with_zone (temp, icaltimezone_get_utc_timezone ());
-	gmtime_r (&current_time, &tm);
+	/* Server utc time is the time when we lasted updated changes from server. local_utc_time is the system utc time. As
+	   there is no way to get the server time on demand, we use the store system utc time to calculate server's time */
+	local_utc_time = e_cal_backend_store_get_key_value (store, LOCAL_UTC_TIME);
+	current = icaltime_current_time_with_zone (icaltimezone_get_utc_timezone ());
 
-	time_interval = get_cache_refresh_interval (cbgw) / 60000;
+	if (priv->first_delta_fetch || !local_utc_time || !*local_utc_time) {
+		e_cal_backend_store_put_key_value (store, SERVER_UTC_TIME, e_gw_connection_get_server_time (cnc));
+	} else {
+		icaltimetype old_local, server_utc;
+		struct icaldurationtype dur;
+
+		old_local = icaltime_from_string (local_utc_time);
+
+		dur = icaltime_subtract (current, old_local);
+		server_utc = icaltime_from_string (time_string);
+		icaltime_adjust (&server_utc, dur.days, dur.hours, dur.minutes, dur.seconds);
+
+		current_time = icaltime_as_timet_with_zone (server_utc, icaltimezone_get_utc_timezone ());
+		gmtime_r (&current_time, &tm);
+
+		strftime (t_str, 26, "%Y-%m-%dT%H:%M:%SZ", &tm);
+		time_string = t_str;
+
+		e_cal_backend_store_put_key_value (store, SERVER_UTC_TIME, time_string);
+	}
+
+	priv->first_delta_fetch = FALSE;
+
+	current_time = icaltime_as_timet_with_zone (current, icaltimezone_get_utc_timezone ());
+	gmtime_r (&current_time, &tm);
+	strftime (t_str, 26, "%Y-%m-%dT%H:%M:%SZ", &tm);
+	time_string = t_str;
+	e_cal_backend_store_put_key_value (store, LOCAL_UTC_TIME, time_string);
 
 	if (attempts) {
-		tm.tm_min += (time_interval * g_ascii_strtod (attempts, NULL));
 		e_cal_backend_store_put_key_value (store, ATTEMPTS_KEY, NULL);
-	} else {
-		tm.tm_min += time_interval;
+		g_free (attempts);
 	}
-	g_free (attempts);
-	strftime (t_str, 26, "%Y-%m-%dT%H:%M:%SZ", &tm);
-	time_string = g_strdup (t_str);
-
-	e_cal_backend_store_put_key_value (store, SERVER_UTC_TIME, time_string);
-
-	g_free (time_string);
-	time_string = NULL;
 
 	if (item_list) {
 		g_list_free (item_list);
@@ -653,8 +687,7 @@ get_deltas (gpointer handle)
 			comp = e_gw_item_to_cal_component (item, cbgw);
 			if (comp) {
 				e_cal_component_commit_sequence (comp);
-				e_cal_backend_store_put_component (store, comp);
-
+				put_component_to_store (cbgw, comp);
 				if (kind == icalcomponent_isa (e_cal_component_get_icalcomponent (comp))) {
 					tmp = e_cal_component_get_as_string (comp);
 					e_cal_backend_notify_object_created (E_CAL_BACKEND (cbgw), tmp);
@@ -990,6 +1023,9 @@ connect_to_server (ECalBackendGroupwise *cbgw, GError **perror)
 		return;
 	}
 
+	errors.status = E_GW_CONNECTION_STATUS_OK;
+	errors.description = NULL;
+
 	kind = e_cal_backend_get_kind (E_CAL_BACKEND (cbgw));
 
 	parent_user = (gchar *) e_source_get_property (source, "parent_id_name");
@@ -1054,9 +1090,14 @@ connect_to_server (ECalBackendGroupwise *cbgw, GError **perror)
 		if (errors.status == E_GW_CONNECTION_STATUS_INVALID_PASSWORD) {
 			g_propagate_error (perror, EDC_ERROR (AuthenticationFailed));
 			return;
+		} else if (errors.status == E_GW_CONNECTION_STATUS_UNKNOWN) {
+			g_propagate_error (perror,EDC_ERROR (OtherError));
+			return;
 		}
 
 		g_propagate_error (perror, EDC_ERROR_EX (OtherError, _(errors.description)));
+		if (errors.description)
+			g_free (errors.description);
 		return;
 	}
 	priv->mode_changed = FALSE;
@@ -1084,7 +1125,6 @@ connect_to_server (ECalBackendGroupwise *cbgw, GError **perror)
 		}
 
 		e_cal_backend_store_load (priv->store);
-		e_cal_backend_store_set_default_timezone (priv->store, priv->default_zone);
 
 		/* spawn a new thread for opening the calendar */
 		thread = g_thread_create ((GThreadFunc) cache_init, cbgw, FALSE, &error);
@@ -1345,8 +1385,7 @@ e_cal_backend_groupwise_open (ECalBackendSync *backend, EDataCal *cal, gboolean
 			}
 		}
 
-		e_cal_backend_store_set_default_timezone (priv->store, priv->default_zone);
-
+		e_cal_backend_store_load (priv->store);
 		PRIV_UNLOCK (priv);
 		return;
 	}
@@ -1597,25 +1636,31 @@ e_cal_backend_groupwise_get_object_list (ECalBackendSync *backend, EDataCal *cal
         GSList *components, *l;
 	ECalBackendSExp *cbsexp;
 	gboolean search_needed = TRUE;
+	time_t occur_start = -1, occur_end = -1;
+	gboolean prunning_by_time;
 
 	cbgw = E_CAL_BACKEND_GROUPWISE (backend);
 	priv = cbgw->priv;
 
-	PRIV_LOCK (priv);
-
 	if (!strcmp (sexp, "#t"))
 		search_needed = FALSE;
 
 	cbsexp = e_cal_backend_sexp_new (sexp);
 	if (!cbsexp) {
-		PRIV_UNLOCK (priv);
 		g_propagate_error (perror, EDC_ERROR (InvalidQuery));
 		return;
 	}
 
 	*objects = NULL;
-	components = e_cal_backend_store_get_components (priv->store);
-        for (l = components; l != NULL; l = l->next) {
+
+	prunning_by_time = e_cal_backend_sexp_evaluate_occur_times(cbsexp,
+									    &occur_start,
+									    &occur_end);
+	components = prunning_by_time ?
+		e_cal_backend_store_get_components_occuring_in_range (priv->store, occur_start, occur_end)
+		: e_cal_backend_store_get_components (priv->store);
+
+	for (l = components; l != NULL; l = l->next) {
                 ECalComponent *comp = E_CAL_COMPONENT (l->data);
 
 		if (e_cal_backend_get_kind (E_CAL_BACKEND (backend)) ==
@@ -1630,8 +1675,6 @@ e_cal_backend_groupwise_get_object_list (ECalBackendSync *backend, EDataCal *cal
 	g_object_unref (cbsexp);
 	g_slist_foreach (components, (GFunc) g_object_unref, NULL);
 	g_slist_free (components);
-
-	PRIV_UNLOCK (priv);
 }
 
 /* get_query handler for the groupwise backend */
@@ -1876,7 +1919,7 @@ update_from_server (ECalBackendGroupwise *cbgw, GSList *uid_list, gchar **calobj
 		item = (EGwItem *) tmp->data;
 		e_cal_comp = e_gw_item_to_cal_component (item, cbgw);
 		e_cal_component_commit_sequence (e_cal_comp);
-		e_cal_backend_store_put_component (priv->store, e_cal_comp);
+		put_component_to_store (cbgw, e_cal_comp);
 
 		if (i == 0) {
 			*calobj = e_cal_component_get_as_string (e_cal_comp);
@@ -1961,7 +2004,6 @@ e_cal_backend_groupwise_create_object (ECalBackendSync *backend, EDataCal *cal,
 
 		/* If delay deliver has been set, server will not send the uid */
 		if (!uid_list || ((e_cal_component_get_vtype (comp) == E_CAL_COMPONENT_JOURNAL) && e_cal_component_has_organizer (comp))) {
-			*calobj = NULL;
 			g_object_unref (comp);
 			return;
 		}
@@ -2092,7 +2134,7 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend, EDataCal *cal,
 				return;
 			}
 
-			e_cal_backend_store_put_component (priv->store, comp);
+			put_component_to_store (cbgw, comp);
 			*new_object = e_cal_component_get_as_string (comp);
 			break;
 		}
@@ -2124,7 +2166,7 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend, EDataCal *cal,
 					g_propagate_error (error, EDC_ERROR_FAILED_STATUS (OtherError, status));
 					return;
 				}
-				e_cal_backend_store_put_component (priv->store, comp);
+				put_component_to_store (cbgw, comp);
 				break;
 			}
 		}
@@ -2148,7 +2190,7 @@ e_cal_backend_groupwise_modify_object (ECalBackendSync *backend, EDataCal *cal,
 
 	case CAL_MODE_LOCAL :
 		/* in offline mode, we just update the cache */
-		e_cal_backend_store_put_component (priv->store, comp);
+		put_component_to_store (cbgw, comp);
 		break;
 	default :
 		break;
@@ -2524,8 +2566,7 @@ receive_object (ECalBackendGroupwise *cbgw, EDataCal *cal, icalcomponent *icalco
 				change_status (component, pstatus, e_gw_connection_get_user_email (priv->cnc));
 				e_cal_component_get_transparency (comp, &transp);
 				e_cal_component_set_transparency (component, transp);
-
-				e_cal_backend_store_put_component (priv->store, component);
+				put_component_to_store (cbgw, comp);
 				comp_str = e_cal_component_get_as_string (component);
 
 				if (found)
@@ -2744,6 +2785,7 @@ e_cal_backend_groupwise_init (ECalBackendGroupwise *cbgw)
 
 	priv->cnc = NULL;
 	priv->sendoptions_sync_timeout = 0;
+	priv->first_delta_fetch = TRUE;
 
 	/* create the mutex for thread safety */
 	g_static_rec_mutex_init (&priv->rec_mutex);
