@@ -1,17 +1,15 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 
-#include <bonobo/bonobo-main.h>
 #include <stdlib.h>
 #include <libecal/e-cal.h>
 
-int
-main (int argc, char **argv)
+gint
+main (gint argc, gchar **argv)
 {
 	ECal *ecal;
 	GList *l, *objects;
 
-	if (bonobo_init (&argc, argv) == FALSE)
-		g_error ("Could not initialize Bonobo");
+	g_type_init ();
 
 	if (argc < 3) {
 		printf ("usage: test-search <uid> <query>\n");
@@ -33,8 +31,8 @@ main (int argc, char **argv)
 	printf ("Received %d objects\n", g_list_length (objects));
 	for (l = objects; l; l = l->next) {
 		ECalComponent *comp = E_CAL_COMPONENT (l->data);
-		char *str;
-		
+		gchar *str;
+
 		str = e_cal_component_get_as_string (comp);
 		printf ("%s\n", str);
 
