@@ -62,7 +62,7 @@ typedef struct _CamelDataWrapperPrivate CamelDataWrapperPrivate;
 /**
  * CamelDataWrapperLock:
  *
- * Since: 3.0
+ * Since: 2.32
  **/
 typedef enum {
 	CAMEL_DATA_WRAPPER_STREAM_LOCK
@@ -91,11 +91,14 @@ struct _CamelDataWrapperClass {
 	void		(*set_mime_type_field)	(CamelDataWrapper *data_wrapper,
 						 CamelContentType *mime_type_field);
 	gssize		(*write_to_stream)	(CamelDataWrapper *data_wrapper,
-						 CamelStream *stream);
+						 CamelStream *stream,
+						 GError **error);
 	gssize		(*decode_to_stream)	(CamelDataWrapper *data_wrapper,
-						 CamelStream *stream);
+						 CamelStream *stream,
+						 GError **error);
 	gint		(*construct_from_stream)(CamelDataWrapper *data_wrapper,
-						 CamelStream *);
+						 CamelStream *stream,
+						 GError **error);
 	gboolean	(*is_offline)		(CamelDataWrapper *data_wrapper);
 };
 
@@ -104,10 +107,12 @@ CamelDataWrapper *
 		camel_data_wrapper_new		(void);
 gssize		camel_data_wrapper_write_to_stream
 						(CamelDataWrapper *data_wrapper,
-						 CamelStream *stream);
+						 CamelStream *stream,
+						 GError **error);
 gssize		camel_data_wrapper_decode_to_stream
 						(CamelDataWrapper *data_wrapper,
-						 CamelStream *stream);
+						 CamelStream *stream,
+						 GError **error);
 void		camel_data_wrapper_set_mime_type(CamelDataWrapper *data_wrapper,
 						 const gchar *mime_type);
 gchar *		camel_data_wrapper_get_mime_type(CamelDataWrapper *data_wrapper);
@@ -119,7 +124,8 @@ void		camel_data_wrapper_set_mime_type_field
 						 CamelContentType *mime_type);
 gint		camel_data_wrapper_construct_from_stream
 						(CamelDataWrapper *data_wrapper,
-						 CamelStream *stream);
+						 CamelStream *stream,
+						 GError **error);
 gboolean	camel_data_wrapper_is_offline	(CamelDataWrapper *data_wrapper);
 void		camel_data_wrapper_lock		(CamelDataWrapper *data_wrapper,
 						 CamelDataWrapperLock lock);
