@@ -48,7 +48,6 @@
 #include <glib/gi18n-lib.h>
 
 #include "camel-data-wrapper.h"
-#include "camel-exception.h"
 #include "camel-mime-filter-basic.h"
 #include "camel-mime-filter-canon.h"
 #include "camel-mime-part.h"
@@ -98,7 +97,7 @@ smime_cert_data_clone (gpointer cert_data)
 static void
 sm_write_stream (gpointer arg, const gchar *buf, gulong len)
 {
-	camel_stream_write ((CamelStream *)arg, buf, len);
+	camel_stream_write ((CamelStream *)arg, buf, len, NULL);
 }
 
 static PK11SymKey *
@@ -295,7 +294,7 @@ nss_error_to_string (glong errorcode)
 }
 
 static void
-set_nss_error (CamelException *ex,
+set_nss_error (GError **error,
                const gchar *def_error)
 {
 	glong err_code;
@@ -305,8 +304,10 @@ set_nss_error (CamelException *ex,
 	err_code = PORT_GetError ();
 
 	if (!err_code) {
-		camel_exception_set (
-			ex, CAMEL_EXCEPTION_SYSTEM, def_error);
+		g_set_error (
+			error, CAMEL_SERVICE_ERROR,
+			CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
+			"%s", def_error);
 	} else {
 		const gchar *err_str;
 
@@ -314,8 +315,9 @@ set_nss_error (CamelException *ex,
 		if (!err_str)
 			err_str = "Uknown error.";
 
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_SYSTEM,
+		g_set_error (
+			error, CAMEL_SERVICE_ERROR,
+			CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
 			"%s (%d) - %s", err_str, (gint) err_code, def_error);
 	}
 }
@@ -323,9 +325,9 @@ set_nss_error (CamelException *ex,
 static NSSCMSMessage *
 sm_signing_cmsmessage (CamelSMIMEContext *context,
                        const gchar *nick,
-                       SECOidTag hash,
+                       SECOidTag *hash,
                        gint detached,
-                       CamelException *ex)
+                       GError **error)
 {
 	CamelSMIMEContextPrivate *p = context->priv;
 	NSSCMSMessage *cmsg = NULL;
@@ -334,63 +336,87 @@ sm_signing_cmsmessage (CamelSMIMEContext *context,
 	NSSCMSSignerInfo *signerinfo;
 	CERTCertificate *cert= NULL, *ekpcert = NULL;
 
+	g_return_val_if_fail (hash != NULL, NULL);
+
 	if ((cert = CERT_FindUserCertByUsage (p->certdb,
 					     (gchar *)nick,
 					     certUsageEmailSigner,
 					     PR_TRUE,
 					     NULL)) == NULL) {
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_SYSTEM,
+		g_set_error (
+			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 			_("Cannot find certificate for '%s'"), nick);
 		return NULL;
 	}
 
+	if (*hash == SEC_OID_UNKNOWN) {
+		/* use signature algorithm from the certificate */
+		switch (SECOID_GetAlgorithmTag (&cert->signature)) {
+		case SEC_OID_PKCS1_SHA256_WITH_RSA_ENCRYPTION:
+			*hash = SEC_OID_SHA256;
+			break;
+		case SEC_OID_PKCS1_SHA384_WITH_RSA_ENCRYPTION:
+			*hash = SEC_OID_SHA384;
+			break;
+		case SEC_OID_PKCS1_SHA512_WITH_RSA_ENCRYPTION:
+			*hash = SEC_OID_SHA512;
+			break;
+		case SEC_OID_PKCS1_MD5_WITH_RSA_ENCRYPTION:
+			*hash = SEC_OID_MD5;
+			break;
+		case SEC_OID_PKCS1_SHA1_WITH_RSA_ENCRYPTION:
+		default:
+			*hash = SEC_OID_SHA1;
+			break;
+		}
+	}
+
 	cmsg = NSS_CMSMessage_Create (NULL); /* create a message on its own pool */
 	if (cmsg == NULL) {
-		set_nss_error (ex, _("Cannot create CMS message"));
+		set_nss_error (error, _("Cannot create CMS message"));
 		goto fail;
 	}
 
 	if ((sigd = NSS_CMSSignedData_Create (cmsg)) == NULL) {
-		set_nss_error (ex, _("Cannot create CMS signed data"));
+		set_nss_error (error, _("Cannot create CMS signed data"));
 		goto fail;
 	}
 
 	cinfo = NSS_CMSMessage_GetContentInfo (cmsg);
 	if (NSS_CMSContentInfo_SetContent_SignedData (cmsg, cinfo, sigd) != SECSuccess) {
-		set_nss_error (ex, _("Cannot attach CMS signed data"));
+		set_nss_error (error, _("Cannot attach CMS signed data"));
 		goto fail;
 	}
 
 	/* if !detatched, the contentinfo will alloc a data item for us */
 	cinfo = NSS_CMSSignedData_GetContentInfo (sigd);
 	if (NSS_CMSContentInfo_SetContent_Data (cmsg, cinfo, NULL, detached) != SECSuccess) {
-		set_nss_error (ex, _("Cannot attach CMS data"));
+		set_nss_error (error, _("Cannot attach CMS data"));
 		goto fail;
 	}
 
-	signerinfo = NSS_CMSSignerInfo_Create (cmsg, cert, hash);
+	signerinfo = NSS_CMSSignerInfo_Create (cmsg, cert, *hash);
 	if (signerinfo == NULL) {
-		set_nss_error (ex, _("Cannot create CMS Signer information"));
+		set_nss_error (error, _("Cannot create CMS Signer information"));
 		goto fail;
 	}
 
 	/* we want the cert chain included for this one */
 	if (NSS_CMSSignerInfo_IncludeCerts (signerinfo, NSSCMSCM_CertChain, certUsageEmailSigner) != SECSuccess) {
-		set_nss_error (ex, _("Cannot find certificate chain"));
+		set_nss_error (error, _("Cannot find certificate chain"));
 		goto fail;
 	}
 
 	/* SMIME RFC says signing time should always be added */
 	if (NSS_CMSSignerInfo_AddSigningTime (signerinfo, PR_Now ()) != SECSuccess) {
-		set_nss_error (ex, _("Cannot add CMS Signing time"));
+		set_nss_error (error, _("Cannot add CMS Signing time"));
 		goto fail;
 	}
 
 #if 0
 	/* this can but needn't be added.  not sure what general usage is */
 	if (NSS_CMSSignerInfo_AddSMIMECaps (signerinfo) != SECSuccess) {
-		fprintf(stderr, "ERROR: cannot add SMIMECaps attribute.\n");
+		fprintf (stderr, "ERROR: cannot add SMIMECaps attribute.\n");
 		goto loser;
 	}
 #endif
@@ -401,12 +427,12 @@ sm_signing_cmsmessage (CamelSMIMEContext *context,
 
 		if (p->encrypt_key) {
 			/* encrypt key has its own nick */
-			if ((ekpcert = CERT_FindUserCertByUsage(
+			if ((ekpcert = CERT_FindUserCertByUsage (
 				     p->certdb,
 				     p->encrypt_key,
 				     certUsageEmailRecipient, PR_TRUE, NULL)) == NULL) {
-				camel_exception_setv (
-					ex, CAMEL_EXCEPTION_SYSTEM,
+				g_set_error (
+					error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 					_("Encryption certificate for '%s' does not exist"),
 					p->encrypt_key);
 				goto fail;
@@ -420,8 +446,8 @@ sm_signing_cmsmessage (CamelSMIMEContext *context,
 			if ((ekpcert = CERT_FindUserCertByUsage (
 				     p->certdb, (gchar *)nick,
 				     certUsageEmailRecipient, PR_TRUE, NULL)) == NULL) {
-				camel_exception_setv (
-					ex, CAMEL_EXCEPTION_SYSTEM,
+				g_set_error (
+					error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
 					_("Encryption certificate for '%s' does not exist"), nick);
 				goto fail;
 			}
@@ -429,23 +455,23 @@ sm_signing_cmsmessage (CamelSMIMEContext *context,
 		}
 
 		if (NSS_CMSSignerInfo_AddSMIMEEncKeyPrefs (signerinfo, enccert, p->certdb) != SECSuccess) {
-			set_nss_error (ex, _("Cannot add SMIMEEncKeyPrefs attribute"));
+			set_nss_error (error, _("Cannot add SMIMEEncKeyPrefs attribute"));
 			goto fail;
 		}
 
 		if (NSS_CMSSignerInfo_AddMSSMIMEEncKeyPrefs (signerinfo, enccert, p->certdb) != SECSuccess) {
-			set_nss_error (ex, _("Cannot add MS SMIMEEncKeyPrefs attribute"));
+			set_nss_error (error, _("Cannot add MS SMIMEEncKeyPrefs attribute"));
 			goto fail;
 		}
 
 		if (ekpcert != NULL && NSS_CMSSignedData_AddCertificate (sigd, ekpcert) != SECSuccess) {
-			set_nss_error (ex, _("Cannot add encryption certificate"));
+			set_nss_error (error, _("Cannot add encryption certificate"));
 			goto fail;
 		}
 	}
 
 	if (NSS_CMSSignedData_AddSignerInfo (sigd, signerinfo) != SECSuccess) {
-		set_nss_error (ex, _("Cannot add CMS Signer information"));
+		set_nss_error (error, _("Cannot add CMS Signer information"));
 		goto fail;
 	}
 
@@ -503,7 +529,7 @@ static CamelCipherValidity *
 sm_verify_cmsg (CamelCipherContext *context,
                 NSSCMSMessage *cmsg,
                 CamelStream *extstream,
-                CamelException *ex)
+                GError **error)
 {
 	CamelSMIMEContextPrivate *p = ((CamelSMIMEContext *)context)->priv;
 	NSSCMSSignedData *sigd = NULL;
@@ -537,17 +563,17 @@ sm_verify_cmsg (CamelCipherContext *context,
 		case SEC_OID_PKCS7_SIGNED_DATA:
 			sigd = (NSSCMSSignedData *)NSS_CMSContentInfo_GetContent (cinfo);
 			if (sigd == NULL) {
-				set_nss_error (ex, _("No signed data in signature"));
+				set_nss_error (error, _("No signed data in signature"));
 				goto fail;
 			}
 
 			if (extstream == NULL) {
-				set_nss_error (ex, _("Digests missing from enveloped data"));
+				set_nss_error (error, _("Digests missing from enveloped data"));
 				goto fail;
 			}
 
 			if ((poolp = PORT_NewArena (1024)) == NULL) {
-				set_nss_error (ex, g_strerror (ENOMEM));
+				set_nss_error (error, g_strerror (ENOMEM));
 				goto fail;
 			}
 
@@ -555,29 +581,29 @@ sm_verify_cmsg (CamelCipherContext *context,
 
 			digcx = NSS_CMSDigestContext_StartMultiple (digestalgs);
 			if (digcx == NULL) {
-				set_nss_error (ex, _("Cannot calculate digests"));
+				set_nss_error (error, _("Cannot calculate digests"));
 				goto fail;
 			}
 
 			buffer = g_byte_array_new ();
 			mem = camel_stream_mem_new_with_byte_array (buffer);
-			camel_stream_write_to_stream (extstream, mem);
+			camel_stream_write_to_stream (extstream, mem, NULL);
 			NSS_CMSDigestContext_Update (digcx, buffer->data, buffer->len);
 			g_object_unref (mem);
 
 			if (NSS_CMSDigestContext_FinishMultiple (digcx, poolp, &digests) != SECSuccess) {
-				set_nss_error (ex, _("Cannot calculate digests"));
+				set_nss_error (error, _("Cannot calculate digests"));
 				goto fail;
 			}
 
 			for (which_digest = 0; digests[which_digest] != NULL; which_digest++) {
 				SECOidData *digest_alg = SECOID_FindOID (&digestalgs[which_digest]->algorithm);
 				if (digest_alg == NULL) {
-					set_nss_error (ex, _("Cannot set message digests"));
+					set_nss_error (error, _("Cannot set message digests"));
 					goto fail;
 				}
 				if (NSS_CMSSignedData_SetDigestValue (sigd, digest_alg->offset, digests[which_digest]) != SECSuccess) {
-					set_nss_error (ex, _("Cannot set message digests"));
+					set_nss_error (error, _("Cannot set message digests"));
 					goto fail;
 				}
 			}
@@ -587,12 +613,12 @@ sm_verify_cmsg (CamelCipherContext *context,
 
 			/* import all certificates present */
 			if (NSS_CMSSignedData_ImportCerts (sigd, p->certdb, certUsageEmailSigner, PR_TRUE) != SECSuccess) {
-				set_nss_error (ex, _("Certificate import failed"));
+				set_nss_error (error, _("Certificate import failed"));
 				goto fail;
 			}
 
 			if (NSS_CMSSignedData_ImportCerts (sigd, p->certdb, certUsageEmailRecipient, PR_TRUE) != SECSuccess) {
-				set_nss_error (ex, _("Certificate import failed"));
+				set_nss_error (error, _("Certificate import failed"));
 				goto fail;
 			}
 
@@ -609,7 +635,7 @@ sm_verify_cmsg (CamelCipherContext *context,
 				}
 			} else {
 				if (!NSS_CMSSignedData_HasDigests (sigd)) {
-					set_nss_error (ex, _("Cannot find signature digests"));
+					set_nss_error (error, _("Cannot find signature digests"));
 					goto fail;
 				}
 
@@ -680,6 +706,12 @@ smime_context_hash_to_id (CamelCipherContext *context,
 	case CAMEL_CIPHER_HASH_SHA1:
 	case CAMEL_CIPHER_HASH_DEFAULT:
 		return "sha1";
+	case CAMEL_CIPHER_HASH_SHA256:
+		return "sha256";
+	case CAMEL_CIPHER_HASH_SHA384:
+		return "sha384";
+	case CAMEL_CIPHER_HASH_SHA512:
+		return "sha512";
 	default:
 		return NULL;
 	}
@@ -694,6 +726,33 @@ smime_context_id_to_hash (CamelCipherContext *context,
 			return CAMEL_CIPHER_HASH_MD5;
 		else if (!strcmp (id, "sha1"))
 			return CAMEL_CIPHER_HASH_SHA1;
+		else if (!strcmp (id, "sha256"))
+			return CAMEL_CIPHER_HASH_SHA256;
+		else if (!strcmp (id, "sha384"))
+			return CAMEL_CIPHER_HASH_SHA384;
+		else if (!strcmp (id, "sha512"))
+			return CAMEL_CIPHER_HASH_SHA512;
+	}
+
+	return CAMEL_CIPHER_HASH_DEFAULT;
+}
+
+static CamelCipherHash
+get_hash_from_oid (SECOidTag oidTag)
+{
+	switch (oidTag) {
+	case SEC_OID_SHA1:
+		return CAMEL_CIPHER_HASH_SHA1;
+	case SEC_OID_SHA256:
+		return CAMEL_CIPHER_HASH_SHA256;
+	case SEC_OID_SHA384:
+		return CAMEL_CIPHER_HASH_SHA384;
+	case SEC_OID_SHA512:
+		return CAMEL_CIPHER_HASH_SHA512;
+	case SEC_OID_MD5:
+		return CAMEL_CIPHER_HASH_MD5;
+	default:
+		break;
 	}
 
 	return CAMEL_CIPHER_HASH_DEFAULT;
@@ -705,7 +764,7 @@ smime_context_sign (CamelCipherContext *context,
                     CamelCipherHash hash,
                     CamelMimePart *ipart,
                     CamelMimePart *opart,
-                    CamelException *ex)
+                    GError **error)
 {
 	CamelCipherContextClass *class;
 	gint res = -1;
@@ -720,19 +779,30 @@ smime_context_sign (CamelCipherContext *context,
 	class = CAMEL_CIPHER_CONTEXT_GET_CLASS (context);
 
 	switch (hash) {
-	case CAMEL_CIPHER_HASH_SHA1:
 	case CAMEL_CIPHER_HASH_DEFAULT:
 	default:
+		sechash = SEC_OID_UNKNOWN;
+		break;
+	case CAMEL_CIPHER_HASH_SHA1:
 		sechash = SEC_OID_SHA1;
 		break;
+	case CAMEL_CIPHER_HASH_SHA256:
+		sechash = SEC_OID_SHA256;
+		break;
+	case CAMEL_CIPHER_HASH_SHA384:
+		sechash = SEC_OID_SHA384;
+		break;
+	case CAMEL_CIPHER_HASH_SHA512:
+		sechash = SEC_OID_SHA512;
+		break;
 	case CAMEL_CIPHER_HASH_MD5:
 		sechash = SEC_OID_MD5;
 		break;
 	}
 
 	cmsg = sm_signing_cmsmessage (
-		(CamelSMIMEContext *)context, userid, sechash,
-		((CamelSMIMEContext *)context)->priv->sign_mode == CAMEL_SMIME_SIGN_CLEARSIGN, ex);
+		(CamelSMIMEContext *)context, userid, &sechash,
+		((CamelSMIMEContext *)context)->priv->sign_mode == CAMEL_SMIME_SIGN_CLEARSIGN, error);
 	if (cmsg == NULL)
 		return -1;
 
@@ -746,10 +816,9 @@ smime_context_sign (CamelCipherContext *context,
 		ipart, CAMEL_MIME_FILTER_CANON_STRIP |
 		CAMEL_MIME_FILTER_CANON_CRLF |
 		CAMEL_MIME_FILTER_CANON_FROM,
-		istream) == -1) {
-		camel_exception_setv (
-			ex, CAMEL_EXCEPTION_SYSTEM,
-			_("Could not generate signing data: %s"), g_strerror(errno));
+		istream, error) == -1) {
+		g_prefix_error (
+			error, _("Could not generate signing data: "));
 		goto fail;
 	}
 
@@ -760,26 +829,26 @@ smime_context_sign (CamelCipherContext *context,
 				   NULL, NULL,     /* decrypt key callback */
 				   NULL, NULL );   /* detached digests    */
 	if (!enc) {
-		set_nss_error (ex, _("Cannot create encoder context"));
+		set_nss_error (error, _("Cannot create encoder context"));
 		goto fail;
 	}
 
 	if (NSS_CMSEncoder_Update (enc, (gchar *) buffer->data, buffer->len) != SECSuccess) {
 		NSS_CMSEncoder_Cancel (enc);
-		set_nss_error (ex, _("Failed to add data to CMS encoder"));
+		set_nss_error (error, _("Failed to add data to CMS encoder"));
 		goto fail;
 	}
 
 	if (NSS_CMSEncoder_Finish (enc) != SECSuccess) {
-		set_nss_error (ex, _("Failed to encode data"));
+		set_nss_error (error, _("Failed to encode data"));
 		goto fail;
 	}
 
 	res = 0;
 
 	dw = camel_data_wrapper_new ();
-	camel_stream_reset (ostream);
-	camel_data_wrapper_construct_from_stream (dw, ostream);
+	camel_stream_reset (ostream, NULL);
+	camel_data_wrapper_construct_from_stream (dw, ostream, NULL);
 	dw->encoding = CAMEL_TRANSFER_ENCODING_BINARY;
 
 	if (((CamelSMIMEContext *)context)->priv->sign_mode == CAMEL_SMIME_SIGN_CLEARSIGN) {
@@ -800,7 +869,7 @@ smime_context_sign (CamelCipherContext *context,
 
 		mps = camel_multipart_signed_new ();
 		ct = camel_content_type_new ("multipart", "signed");
-		camel_content_type_set_param (ct, "micalg", camel_cipher_hash_to_id (context, hash));
+		camel_content_type_set_param (ct, "micalg", camel_cipher_hash_to_id (context, get_hash_from_oid (sechash)));
 		camel_content_type_set_param (ct, "protocol", class->sign_protocol);
 		camel_data_wrapper_set_mime_type_field ((CamelDataWrapper *)mps, ct);
 		camel_content_type_unref (ct);
@@ -808,7 +877,7 @@ smime_context_sign (CamelCipherContext *context,
 
 		mps->signature = sigpart;
 		mps->contentraw = istream;
-		camel_stream_reset (istream);
+		camel_stream_reset (istream, NULL);
 		g_object_ref (istream);
 
 		camel_medium_set_content ((CamelMedium *)opart, (CamelDataWrapper *)mps);
@@ -838,7 +907,7 @@ fail:
 static CamelCipherValidity *
 smime_context_verify (CamelCipherContext *context,
                       CamelMimePart *ipart,
-                      CamelException *ex)
+                      GError **error)
 {
 	CamelCipherContextClass *class;
 	NSSCMSDecoderContext *dec;
@@ -869,30 +938,33 @@ smime_context_verify (CamelCipherContext *context,
 		    || tmp == NULL
 		    || (g_ascii_strcasecmp (tmp, class->sign_protocol) != 0
 			&& g_ascii_strcasecmp (tmp, "application/pkcs7-signature") != 0)) {
-			camel_exception_set (
-				ex, CAMEL_EXCEPTION_SYSTEM,
-				_("Cannot verify message signature: Incorrect message format"));
+			g_set_error (
+				error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+				_("Cannot verify message signature: "
+				  "Incorrect message format"));
 			goto fail;
 		}
 
 		constream = camel_multipart_signed_get_content_stream (
-			(CamelMultipartSigned *)mps, ex);
+			(CamelMultipartSigned *)mps, error);
 		if (constream == NULL)
 			goto fail;
 
 		sigpart = camel_multipart_get_part (mps, CAMEL_MULTIPART_SIGNED_SIGNATURE);
 		if (sigpart == NULL) {
-			camel_exception_set (
-				ex, CAMEL_EXCEPTION_SYSTEM,
-				_("Cannot verify message signature: Incorrect message format"));
+			g_set_error (
+				error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+				_("Cannot verify message signature: "
+				  "Incorrect message format"));
 			goto fail;
 		}
 	} else if (camel_content_type_is (ct, "application", "x-pkcs7-mime")) {
 		sigpart = ipart;
 	} else {
-		camel_exception_set (
-			ex, CAMEL_EXCEPTION_SYSTEM,
-			_("Cannot verify message signature: Incorrect message format"));
+		g_set_error (
+			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+			_("Cannot verify message signature: "
+			  "Incorrect message format"));
 		goto fail;
 	}
 
@@ -902,15 +974,15 @@ smime_context_verify (CamelCipherContext *context,
 				   NULL, NULL); /* decrypt key callback */
 
 	camel_data_wrapper_decode_to_stream (
-		camel_medium_get_content (CAMEL_MEDIUM (sigpart)), mem);
+		camel_medium_get_content (CAMEL_MEDIUM (sigpart)), mem, NULL);
 	(void)NSS_CMSDecoder_Update (dec, (gchar *) buffer->data, buffer->len);
 	cmsg = NSS_CMSDecoder_Finish (dec);
 	if (cmsg == NULL) {
-		set_nss_error (ex, _("Decoder failed"));
+		set_nss_error (error, _("Decoder failed"));
 		goto fail;
 	}
 
-	valid = sm_verify_cmsg (context, cmsg, constream, ex);
+	valid = sm_verify_cmsg (context, cmsg, constream, error);
 
 	NSS_CMSMessage_Destroy (cmsg);
 fail:
@@ -927,7 +999,7 @@ smime_context_encrypt (CamelCipherContext *context,
                        GPtrArray *recipients,
                        CamelMimePart *ipart,
                        CamelMimePart *opart,
-                       CamelException *ex)
+                       GError **error)
 {
 	CamelSMIMEContextPrivate *p = ((CamelSMIMEContext *)context)->priv;
 	/*NSSCMSRecipientInfo **recipient_infos;*/
@@ -950,30 +1022,31 @@ smime_context_encrypt (CamelCipherContext *context,
 
 	poolp = PORT_NewArena (1024);
 	if (poolp == NULL) {
-		set_nss_error (ex, g_strerror (ENOMEM));
+		set_nss_error (error, g_strerror (ENOMEM));
 		return -1;
 	}
 
 	/* Lookup all recipients certs, for later working */
 	recipient_certs = (CERTCertificate **)PORT_ArenaZAlloc (poolp, sizeof (*recipient_certs[0])*(recipients->len + 1));
 	if (recipient_certs == NULL) {
-		set_nss_error (ex, g_strerror (ENOMEM));
+		set_nss_error (error, g_strerror (ENOMEM));
 		goto fail;
 	}
 
 	for (i=0;i<recipients->len;i++) {
 		recipient_certs[i] = CERT_FindCertByNicknameOrEmailAddr (p->certdb, recipients->pdata[i]);
 		if (recipient_certs[i] == NULL) {
-			camel_exception_setv (
-				ex, CAMEL_EXCEPTION_SYSTEM,
-				_("Cannot find certificate for '%s'"), recipients->pdata[i]);
+			g_set_error (
+				error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+				_("Cannot find certificate for '%s'"),
+				(gchar *) recipients->pdata[i]);
 			goto fail;
 		}
 	}
 
 	/* Find a common algorithm, probably 3DES anyway ... */
 	if (NSS_SMIMEUtil_FindBulkAlgForRecipients (recipient_certs, &bulkalgtag, &bulkkeysize) != SECSuccess) {
-		set_nss_error (ex, _("Cannot find common bulk encryption algorithm"));
+		set_nss_error (error, _("Cannot find common bulk encryption algorithm"));
 		goto fail;
 	}
 
@@ -981,7 +1054,7 @@ smime_context_encrypt (CamelCipherContext *context,
 	type = PK11_AlgtagToMechanism (bulkalgtag);
 	slot = PK11_GetBestSlot (type, context);
 	if (slot == NULL) {
-		set_nss_error (ex, _("Cannot allocate slot for encryption bulk key"));
+		set_nss_error (error, _("Cannot allocate slot for encryption bulk key"));
 		goto fail;
 	}
 
@@ -992,25 +1065,25 @@ smime_context_encrypt (CamelCipherContext *context,
 	/* msg->envelopedData->data */
 	cmsg = NSS_CMSMessage_Create (NULL);
 	if (cmsg == NULL) {
-		set_nss_error (ex, _("Cannot create CMS Message"));
+		set_nss_error (error, _("Cannot create CMS Message"));
 		goto fail;
 	}
 
 	envd = NSS_CMSEnvelopedData_Create (cmsg, bulkalgtag, bulkkeysize);
 	if (envd == NULL) {
-		set_nss_error (ex, _("Cannot create CMS Enveloped data"));
+		set_nss_error (error, _("Cannot create CMS Enveloped data"));
 		goto fail;
 	}
 
 	cinfo = NSS_CMSMessage_GetContentInfo (cmsg);
 	if (NSS_CMSContentInfo_SetContent_EnvelopedData (cmsg, cinfo, envd) != SECSuccess) {
-		set_nss_error (ex, _("Cannot attach CMS Enveloped data"));
+		set_nss_error (error, _("Cannot attach CMS Enveloped data"));
 		goto fail;
 	}
 
 	cinfo = NSS_CMSEnvelopedData_GetContentInfo (envd);
 	if (NSS_CMSContentInfo_SetContent_Data (cmsg, cinfo, NULL, PR_FALSE) != SECSuccess) {
-		set_nss_error (ex, _("Cannot attach CMS data object"));
+		set_nss_error (error, _("Cannot attach CMS data object"));
 		goto fail;
 	}
 
@@ -1019,12 +1092,12 @@ smime_context_encrypt (CamelCipherContext *context,
 		NSSCMSRecipientInfo *ri = NSS_CMSRecipientInfo_Create (cmsg, recipient_certs[i]);
 
 		if (ri == NULL) {
-			set_nss_error (ex, _("Cannot create CMS Recipient information"));
+			set_nss_error (error, _("Cannot create CMS Recipient information"));
 			goto fail;
 		}
 
 		if (NSS_CMSEnvelopedData_AddRecipient (envd, ri) != SECSuccess) {
-			set_nss_error (ex, _("Cannot add CMS Recipient information"));
+			set_nss_error (error, _("Cannot add CMS Recipient information"));
 			goto fail;
 		}
 	}
@@ -1038,24 +1111,24 @@ smime_context_encrypt (CamelCipherContext *context,
 				   sm_decrypt_key, bulkkey,
 				   NULL, NULL);
 	if (enc == NULL) {
-		set_nss_error (ex, _("Cannot create encoder context"));
+		set_nss_error (error, _("Cannot create encoder context"));
 		goto fail;
 	}
 
 	/* FIXME: Stream the input */
 	buffer = g_byte_array_new ();
 	mem = camel_stream_mem_new_with_byte_array (buffer);
-	camel_cipher_canonical_to_stream (ipart, CAMEL_MIME_FILTER_CANON_CRLF, mem);
+	camel_cipher_canonical_to_stream (ipart, CAMEL_MIME_FILTER_CANON_CRLF, mem, NULL);
 	if (NSS_CMSEncoder_Update (enc, (gchar *) buffer->data, buffer->len) != SECSuccess) {
 		NSS_CMSEncoder_Cancel (enc);
 		g_object_unref (mem);
-		set_nss_error (ex, _("Failed to add data to encoder"));
+		set_nss_error (error, _("Failed to add data to encoder"));
 		goto fail;
 	}
 	g_object_unref (mem);
 
-	if (NSS_CMSEncoder_Finish(enc) != SECSuccess) {
-		set_nss_error (ex, _("Failed to encode data"));
+	if (NSS_CMSEncoder_Finish (enc) != SECSuccess) {
+		set_nss_error (error, _("Failed to encode data"));
 		goto fail;
 	}
 
@@ -1066,7 +1139,7 @@ smime_context_encrypt (CamelCipherContext *context,
 	PORT_FreeArena (poolp, PR_FALSE);
 
 	dw = camel_data_wrapper_new ();
-	camel_data_wrapper_construct_from_stream (dw, ostream);
+	camel_data_wrapper_construct_from_stream (dw, ostream, NULL);
 	g_object_unref (ostream);
 	dw->encoding = CAMEL_TRANSFER_ENCODING_BINARY;
 
@@ -1108,7 +1181,7 @@ static CamelCipherValidity *
 smime_context_decrypt (CamelCipherContext *context,
                        CamelMimePart *ipart,
                        CamelMimePart *opart,
-                       CamelException *ex)
+                       GError **error)
 {
 	NSSCMSDecoderContext *dec;
 	NSSCMSMessage *cmsg;
@@ -1126,8 +1199,9 @@ smime_context_decrypt (CamelCipherContext *context,
 	/* FIXME: stream this to the decoder incrementally */
 	buffer = g_byte_array_new ();
 	istream = camel_stream_mem_new_with_byte_array (buffer);
-	camel_data_wrapper_decode_to_stream (camel_medium_get_content ((CamelMedium *)ipart), istream);
-	camel_stream_reset (istream);
+	camel_data_wrapper_decode_to_stream (
+		camel_medium_get_content ((CamelMedium *)ipart), istream, NULL);
+	camel_stream_reset (istream, NULL);
 
 	dec = NSS_CMSDecoder_Start (NULL,
 				   sm_write_stream, ostream, /* content callback     */
@@ -1143,7 +1217,7 @@ smime_context_decrypt (CamelCipherContext *context,
 	g_object_unref (istream);
 
 	if (cmsg == NULL) {
-		set_nss_error (ex, _("Decoder failed"));
+		set_nss_error (error, _("Decoder failed"));
 		goto fail;
 	}
 
@@ -1156,12 +1230,12 @@ smime_context_decrypt (CamelCipherContext *context,
 	}
 #endif
 
-	camel_stream_reset (ostream);
-	camel_data_wrapper_construct_from_stream ((CamelDataWrapper *)opart, ostream);
+	camel_stream_reset (ostream, NULL);
+	camel_data_wrapper_construct_from_stream ((CamelDataWrapper *)opart, ostream, NULL);
 
 	if (NSS_CMSMessage_IsSigned (cmsg)) {
-		camel_stream_reset (ostream);
-		valid = sm_verify_cmsg (context, cmsg, ostream, ex);
+		camel_stream_reset (ostream, NULL);
+		valid = sm_verify_cmsg (context, cmsg, ostream, error);
 	} else {
 		valid = camel_cipher_validity_new ();
 		valid->encrypt.description = g_strdup (_("Encrypted content"));
@@ -1268,8 +1342,10 @@ camel_smime_context_describe_part (CamelSMIMEContext *context, CamelMimePart *pa
 		/* FIXME: stream this to the decoder incrementally */
 		buffer = g_byte_array_new ();
 		istream = camel_stream_mem_new_with_byte_array (buffer);
-		camel_data_wrapper_decode_to_stream (camel_medium_get_content ((CamelMedium *)part), istream);
-		camel_stream_reset (istream);
+		camel_data_wrapper_decode_to_stream (
+			camel_medium_get_content ((CamelMedium *)part),
+			istream, NULL);
+		camel_stream_reset (istream, NULL);
 
 		dec = NSS_CMSDecoder_Start (NULL,
 					   NULL, NULL,
