@@ -88,7 +88,8 @@ stream_mem_finalize (GObject *object)
 static gssize
 stream_mem_read (CamelStream *stream,
                  gchar *buffer,
-                 gsize n)
+                 gsize n,
+                 GError **error)
 {
 	CamelStreamMemPrivate *priv;
 	CamelSeekableStream *seekable = CAMEL_SEEKABLE_STREAM (stream);
@@ -112,7 +113,8 @@ stream_mem_read (CamelStream *stream,
 static gssize
 stream_mem_write (CamelStream *stream,
                   const gchar *buffer,
-                  gsize n)
+                  gsize n,
+                  GError **error)
 {
 	CamelStreamMemPrivate *priv;
 	CamelSeekableStream *seekable = CAMEL_SEEKABLE_STREAM (stream);
@@ -146,13 +148,14 @@ stream_mem_eos (CamelStream *stream)
 	return priv->buffer->len <= seekable_stream->position;
 }
 
-static off_t
+static goffset
 stream_mem_seek (CamelSeekableStream *stream,
-                 off_t offset,
-                 CamelStreamSeekPolicy policy)
+                 goffset offset,
+                 CamelStreamSeekPolicy policy,
+                 GError **error)
 {
 	CamelStreamMemPrivate *priv;
-	off_t position;
+	goffset position;
 
 	priv = CAMEL_STREAM_MEM_GET_PRIVATE (stream);
 
@@ -307,7 +310,7 @@ camel_stream_mem_set_secure(CamelStreamMem *mem)
  * camel_stream_mem_get_byte_array:
  * @mem: a #CamelStreamMem
  *
- * Since: 3.0
+ * Since: 2.32
  **/
 GByteArray *
 camel_stream_mem_get_byte_array (CamelStreamMem *mem)
