@@ -18,39 +18,48 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef _CAMEL_MH_SUMMARY_H
-#define _CAMEL_MH_SUMMARY_H
+#ifndef CAMEL_MH_SUMMARY_H
+#define CAMEL_MH_SUMMARY_H
 
 #include "camel-local-summary.h"
-#include <camel/camel-folder.h>
-#include <camel/camel-exception.h>
-#include <camel/camel-index.h>
 
-#define CAMEL_MH_SUMMARY(obj)	CAMEL_CHECK_CAST (obj, camel_mh_summary_get_type (), CamelMhSummary)
-#define CAMEL_MH_SUMMARY_CLASS(klass)	CAMEL_CHECK_CLASS_CAST (klass, camel_mh_summary_get_type (), CamelMhSummaryClass)
-#define CAMEL_IS_MH_SUMMARY(obj)      CAMEL_CHECK_TYPE (obj, camel_mh_summary_get_type ())
+/* Standard GObject macros */
+#define CAMEL_TYPE_MH_SUMMARY \
+	(camel_mh_summary_get_type ())
+#define CAMEL_MH_SUMMARY(obj) \
+	(G_TYPE_CHECK_INSTANCE_CAST \
+	((obj), CAMEL_TYPE_MH_SUMMARY, CamelMhSummary))
+#define CAMEL_MH_SUMMARY_CLASS(cls) \
+	(G_TYPE_CHECK_CLASS_CAST \
+	((cls), CAMEL_TYPE_MH_SUMMARY, CamelMhSummaryClass))
+#define CAMEL_IS_MH_SUMMARY(obj) \
+	(G_TYPE_CHECK_INSTANCE_TYPE \
+	((obj), CAMEL_TYPE_MH_SUMMARY))
+#define CAMEL_IS_MH_SUMMARY_CLASS(cls) \
+	(G_TYPE_CHECK_CLASS_TYPE \
+	((cls), CAMEL_TYPE_MH_SUMMARY))
+#define CAMEL_MH_SUMMARY_GET_CLASS(obj) \
+	(G_TYPE_INSTANCE_GET_CLASS \
+	((obj), CAMEL_TYPE_MH_SUMMARY, CamelMhSummaryClass))
 
 G_BEGIN_DECLS
 
 typedef struct _CamelMhSummary	CamelMhSummary;
-typedef struct _CamelMhSummaryClass	CamelMhSummaryClass;
+typedef struct _CamelMhSummaryClass CamelMhSummaryClass;
+typedef struct _CamelMhSummaryPrivate CamelMhSummaryPrivate;
 
 struct _CamelMhSummary {
 	CamelLocalSummary parent;
-	struct _CamelMhSummaryPrivate *priv;
+	CamelMhSummaryPrivate *priv;
 };
 
 struct _CamelMhSummaryClass {
 	CamelLocalSummaryClass parent_class;
-
-	/* virtual methods */
-
-	/* signals */
 };
 
-CamelType	 camel_mh_summary_get_type	(void);
+GType	 camel_mh_summary_get_type	(void);
 CamelMhSummary	*camel_mh_summary_new(struct _CamelFolder *, const gchar *filename, const gchar *mhdir, CamelIndex *index);
 
 G_END_DECLS
 
-#endif /* _CAMEL_MH_SUMMARY_H */
+#endif /* CAMEL_MH_SUMMARY_H */
