@@ -9,17 +9,14 @@
 #include <stdio.h>
 #include <iconv.h>
 
-#include <camel/camel-internet-address.h>
-#include <camel/camel-address.h>
-
 #include "address-data.h"
 
-static char *convert(const char *in, const char *from, const char *to)
+static gchar *convert(const gchar *in, const gchar *from, const gchar *to)
 {
 	iconv_t ic = iconv_open(to, from);
-	char *out, *outp;
-	const char *inp;
-	size_t inlen, outlen;
+	gchar *out, *outp;
+	const gchar *inp;
+	gsize inlen, outlen;
 
 	if (ic == (iconv_t)-1)
 		return g_strdup(in);
@@ -49,7 +46,7 @@ static char *convert(const char *in, const char *from, const char *to)
 #if 0
 	/* lets see if we can convert back again? */
 	{
-		char *nout, *noutp;
+		gchar *nout, *noutp;
 		iconv_t ic = iconv_open(from, to);
 
 		if (ic == (iconv_t)-1)
@@ -61,7 +58,7 @@ static char *convert(const char *in, const char *from, const char *to)
 		noutp = nout = g_malloc(outlen);
 		if (iconv(ic, &inp, &inlen, &noutp, &outlen) == -1
 		    || iconv(ic, NULL, 0, &noutp, &outlen) == -1) {
-			g_warning("Cannot convert '%s' \n from %s to %s: %s\n", in, to, from, strerror(errno));
+			g_warning("Cannot convert '%s' \n from %s to %s: %s\n", in, to, from, g_strerror(errno));
 		}
 		iconv_close(ic);
 	}
@@ -80,16 +77,14 @@ fail:
 #define to_utf8(in, type) convert(in, type, "utf-8")
 #define from_utf8(in, type) convert(in, "utf-8", type)
 
-#define ARRAY_LEN(x) (sizeof(x)/sizeof(x[0]))
-
-int main(int argc, char **argv)
+gint main(gint argc, gchar **argv)
 {
-	int i;
+	gint i;
 	CamelInternetAddress *addr, *addr2;
-	char *name;
-	char *charset;
-	const char *real, *where;
-	char *enc, *enc2, *format, *format2;
+	gchar *name;
+	gchar *charset;
+	const gchar *real, *where;
+	gchar *enc, *enc2, *format, *format2;
 
 	camel_test_init(argc, argv);
 
@@ -124,7 +119,7 @@ int main(int argc, char **argv)
 
 	push("Test add many");
 	for (i=1;i<10;i++) {
-		char name[16], a[32];
+		gchar name[16], a[32];
 		sprintf(name, "Zed %d", i);
 		sprintf(a, "nowhere@here-%d.com.au", i);
 		camel_internet_address_add(addr, name, a);
@@ -220,7 +215,7 @@ int main(int argc, char **argv)
 
 	camel_test_start("CamelInternetAddress, I18N");
 
-	for (i=0;i<ARRAY_LEN(test_lines);i++) {
+	for (i = 0;i < G_N_ELEMENTS (test_lines); i++) {
 		push("Testing text line %d (%s) '%s'", i, test_lines[i].type, test_lines[i].line);
 
 		addr = camel_internet_address_new();
@@ -295,7 +290,7 @@ int main(int argc, char **argv)
 
 	camel_test_start("CamelInternetAddress, I18N decode");
 
-	for (i=0;i<ARRAY_LEN(test_address);i++) {
+	for (i = 0; i < G_N_ELEMENTS (test_address); i++) {
 		push("Testing address line %d '%s'", i, test_address[i].addr);
 
 		addr = camel_internet_address_new();
@@ -305,7 +300,7 @@ int main(int argc, char **argv)
 		check(strcmp(format, test_address[i].utf8) == 0);
 		test_free(format);
 		pull();
-		
+
 		push("Comparing re-encoded output");
 		addr2 = CAMEL_INTERNET_ADDRESS(camel_internet_address_new());
 		enc = camel_address_encode(CAMEL_ADDRESS(addr));
@@ -327,4 +322,3 @@ int main(int argc, char **argv)
 	return 0;
 }
 
-
