@@ -294,7 +294,16 @@ e_source_group_new_from_xmldoc (xmlDocPtr doc)
 	new->priv->uid = g_strdup (GC uid);
 
 	e_source_group_set_name (new, GC name);
-	e_source_group_set_base_uri (new, GC base_uri);
+
+	/* XXX The "On This Computer" group used to specify an
+	 *     absolute "file:" URI pointing to its local storage
+	 *     directory, but that caused all kinds of portability
+	 *     issues so now we just use "local:" and leave the
+	 *     absolute file system path implicit. */
+	if (g_str_has_prefix (GC base_uri, "file:"))
+		e_source_group_set_base_uri (new, "local:");
+	else
+		e_source_group_set_base_uri (new, GC base_uri);
 
 	for (p = root->children; p != NULL; p = p->next) {
 		ESource *new_source;
@@ -389,7 +398,16 @@ e_source_group_update_from_xmldoc (ESourceGroup *group,
 	}
 	xmlFree (name);
 
-	if (strcmp (group->priv->base_uri, GC base_uri) != 0) {
+	/* XXX The "On This Computer" group used to specify an
+	 *     absolute "file:" URI pointing to its local storage
+	 *     directory, but that caused all kinds of portability
+	 *     issues so now we just use "local:" and leave the
+	 *     absolute file system path implicit. */
+	if (g_str_has_prefix (GC base_uri, "file:")) {
+		g_free (group->priv->base_uri);
+		group->priv->base_uri = g_strdup ("local:");
+		changed = TRUE;
+	} else if (strcmp (group->priv->base_uri, GC base_uri) != 0) {
 		g_free (group->priv->base_uri);
 		group->priv->base_uri = g_strdup (GC base_uri);
 		changed = TRUE;
@@ -457,7 +475,7 @@ e_source_group_update_from_xmldoc (ESourceGroup *group,
 		} else {
 			gboolean source_changed;
 
-			group->priv->ignore_source_changed ++;
+			group->priv->ignore_source_changed++;
 
 			if (e_source_update_from_xml_node (existing_source, nodep, &source_changed)) {
 				new_sources_list = g_slist_prepend (new_sources_list, existing_source);
@@ -468,7 +486,7 @@ e_source_group_update_from_xmldoc (ESourceGroup *group,
 					changed = TRUE;
 			}
 
-			group->priv->ignore_source_changed --;
+			group->priv->ignore_source_changed--;
 		}
 
 		g_free (uid);
@@ -782,7 +800,7 @@ e_source_group_to_xml (ESourceGroup *group)
 
 	returned_buffer = g_malloc (xml_buffer_size + 1);
 	memcpy (returned_buffer, xml_buffer, xml_buffer_size);
-	returned_buffer [xml_buffer_size] = '\0';
+	returned_buffer[xml_buffer_size] = '\0';
 	xmlFree (xml_buffer);
 
 	return returned_buffer;
@@ -820,8 +838,10 @@ compare_source_lists (GSList *a, GSList *b)
  *
  * Compares if @a is equivalent to @b.
  *
- * Return value: %TRUE if @a is equivalent to @b,
+ * Returns: %TRUE if @a is equivalent to @b,
  * %FALSE otherwise.
+ *
+ * Since: 2.24
  **/
 gboolean
 e_source_group_equal (ESourceGroup *a, ESourceGroup *b)
@@ -865,8 +885,10 @@ e_source_group_equal (ESourceGroup *a, ESourceGroup *b)
  *
  * Compares if @a is equivalent to @b.
  *
- * Return value: %TRUE if @a is equivalent to @b,
+ * Returns: %TRUE if @a is equivalent to @b,
  * %FALSE otherwise.
+ *
+ * Since: 2.24
  **/
 gboolean
 e_source_group_xmlstr_equal (const gchar *a, const gchar *b)
@@ -885,6 +907,11 @@ e_source_group_xmlstr_equal (const gchar *a, const gchar *b)
 	return retval;
 }
 
+/**
+ * e_source_group_get_property:
+ *
+ * Since: 1.12
+ **/
 gchar *
 e_source_group_get_property (ESourceGroup *source_group,
 			     const gchar *property)
@@ -897,6 +924,11 @@ e_source_group_get_property (ESourceGroup *source_group,
 	return g_strdup (g_hash_table_lookup (priv->properties, property));
 }
 
+/**
+ * e_source_group_set_property:
+ *
+ * Since: 1.12
+ **/
 void
 e_source_group_set_property (ESourceGroup *source_group,
 			     const gchar *property,
@@ -915,6 +947,11 @@ e_source_group_set_property (ESourceGroup *source_group,
 	g_signal_emit (source_group, signals[CHANGED], 0);
 }
 
+/**
+ * e_source_group_foreach_property:
+ *
+ * Since: 1.12
+ **/
 void
 e_source_group_foreach_property (ESourceGroup *source_group, GHFunc func, gpointer data)
 {
